'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('../_virtual/_rollupPluginBabelHelpers.js');
var etherplex = require('@pooltogether/etherplex');
require('@pooltogether/current-pool-data');
require('./fetchers/getPools.js');
var infura = require('./utils/infura.js');
var quickNode = require('./utils/quickNode.js');
var fetch = require('./utils/fetch.js');

const getRpcUrl = chainId => {
  switch (chainId) {
    case 1:
      return "https://mainnet.infura.io/v3/".concat(infura.INFURA_ID);

    case 4:
      return "https://rinkeby.infura.io/v3/".concat(infura.INFURA_ID);

    case 56:
      return "https://red-fragrant-fire.bsc.quiknode.pro/".concat(quickNode.QUICKNODE_ID, "/");

    case 137:
      return "https://polygon-mainnet.infura.io/v3/".concat(infura.INFURA_ID);

    case 42220:
      return "https://forno.celo.org";

    case 80001:
      return "https://polygon-mumbai.infura.io/v3/".concat(infura.INFURA_ID);
  }
};

const batch = async (chainId, ...batchCalls) => {
  // TODO: currently we assume any PoolTogether pool is on a network which supports multicall
  // so make sure to handle calls differently if the network doesn't support multicall!
  // (see Etherplex for calls fallback for non-multicall networks)
  const _encodeData = etherplex.encodeData(...batchCalls),
        _encodeData2 = _rollupPluginBabelHelpers.slicedToArray(_encodeData, 3),
        result = _encodeData2[0],
        calls = _encodeData2[1],
        data = _encodeData2[2];

  const tx = {
    params: [await etherplex.prepareTransaction(chainId, data), 'latest'],
    jsonrpc: '2.0',
    id: 1,
    method: 'eth_call'
  };
  let callResponse;
  callResponse = await fetch.fetch(getRpcUrl(Number(chainId)), {
    method: 'POST',
    body: JSON.stringify(tx),
    headers: {
      'Content-Type': 'application/json'
    }
  });
  const body = await callResponse.json();
  const decoded = etherplex.decodeData(result, calls, body.result);
  return decoded;
};

exports.batch = batch;
