'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var etherplex = require('@pooltogether/etherplex');
var cloudflareWorkersBatch = require('../cloudflare-workers-batch.js');
var Pod = require('../../abis/Pod.js');
var TokenDropAbi = require('../../abis/TokenDropAbi.js');

const getPodContractAddresses = async (chainId, podAddress) => {
  const podContract = etherplex.contract(podAddress, Pod.PodAbi, podAddress);
  let batchCalls = [];
  batchCalls.push(podContract.owner().name().symbol().decimals().prizePool().manager().faucet().ticket().token().tokenDrop());
  let response = await cloudflareWorkersBatch.batch(chainId, ...batchCalls);
  const podAddresses = {
    address: podAddress,
    owner: response[podAddress].owner[0],
    name: response[podAddress].name[0],
    symbol: response[podAddress].symbol[0],
    decimals: response[podAddress].decimals[0],
    prizePool: response[podAddress].prizePool[0],
    manager: response[podAddress].manager[0],
    faucet: response[podAddress].faucet[0],
    ticket: response[podAddress].ticket[0],
    token: response[podAddress].token[0],
    tokenDrop: response[podAddress].tokenDrop[0]
  };
  const tokenDropAbiContract = etherplex.contract(podAddresses.tokenDrop, TokenDropAbi.TokenDropAbi, podAddresses.tokenDrop);
  response = await cloudflareWorkersBatch.batch(chainId, tokenDropAbiContract.asset());
  podAddresses.tokenDropDripToken = response[podAddresses.tokenDrop].asset[0];
  return podAddresses;
};

exports.getPodContractAddresses = getPodContractAddresses;
