'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('../../_virtual/_rollupPluginBabelHelpers.js');
var currentPoolData = require('@pooltogether/current-pool-data');
var utilities = require('@pooltogether/utilities');
var graphqlRequest = require('graphql-request');
require('../fetchers/getPools.js');
require('@pooltogether/etherplex');
var fetch = require('../utils/fetch.js');

const getSubgraphVersionsFromContracts = poolContracts => [...new Set(poolContracts.map(pool => pool.subgraphVersion))];
const getUniswapSubgraphClient = chainId => new graphqlRequest.GraphQLClient(currentPoolData.UNISWAP_GRAPH_URIS[chainId], {
  fetch: fetch.fetch
});
const getSubgraphClientsByVersionFromContracts = (poolContracts, chainId) => getSubgraphVersionsFromContracts(poolContracts).reduce((accumulator, version) => _rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2({}, accumulator), {}, {
  [version]: new graphqlRequest.GraphQLClient(currentPoolData.POOLTOGETHER_SUBGRAPH_URIS[chainId][version], {
    fetch: fetch.fetch
  })
}), {});
const getPoolAddressesBySubgraphVersionFromContracts = poolContracts => {
  return poolContracts.reduce((accumulator, pool) => {
    return _rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2({}, accumulator), {}, {
      [pool.subgraphVersion]: [].concat(pool.prizePool.address, accumulator !== null && accumulator !== void 0 && accumulator[pool.subgraphVersion] ? accumulator[pool.subgraphVersion] : [])
    });
  }, {});
};
const getLootBoxSubgraphClient = chainId => new graphqlRequest.GraphQLClient(currentPoolData.LOOTBOX_GRAPH_URIS[chainId], {
  fetch: fetch.fetch
});
/**
 * BSC Subgraphs use `derivedBNB` rather than `derivedETH`
 * @param {*} chainId
 * @returns
 */

const getDexNativeCurrencyKey = chainId => {
  switch (chainId) {
    case utilities.NETWORK.bsc:
      return 'derivedBNB';

    case utilities.NETWORK.celo:
      return 'derivedCUSD';

    default:
      return 'derivedETH';
  }
};

exports.getDexNativeCurrencyKey = getDexNativeCurrencyKey;
exports.getLootBoxSubgraphClient = getLootBoxSubgraphClient;
exports.getPoolAddressesBySubgraphVersionFromContracts = getPoolAddressesBySubgraphVersionFromContracts;
exports.getSubgraphClientsByVersionFromContracts = getSubgraphClientsByVersionFromContracts;
exports.getSubgraphVersionsFromContracts = getSubgraphVersionsFromContracts;
exports.getUniswapSubgraphClient = getUniswapSubgraphClient;
