'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var _rollupPluginBabelHelpers = require('../../_virtual/_rollupPluginBabelHelpers.js');
var cloneDeep = require('lodash.clonedeep');
var utilities = require('@pooltogether/utilities');
var units = require('@ethersproject/units');
var ethers = require('ethers');
var currentPoolData = require('@pooltogether/current-pool-data');
var getCustomYieldSourceData = require('../fetchers/getCustomYieldSourceData.js');
require('../fetchers/getPools.js');
require('@pooltogether/etherplex');
var fetch = require('./fetch.js');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var cloneDeep__default = /*#__PURE__*/_interopDefaultLegacy(cloneDeep);

/**
 * Calculates the total yield values, $0 if no yield or no token prices
 * @param {*} _pool
 * @returns
 */

const calculateYieldTotalValuesUsd = async _pool => {
  var _pool$prizePool$yield;

  switch (_pool.prizePool.type) {
    case currentPoolData.PRIZE_POOL_TYPES.compound:
      {
        return await calculateCompoundYieldTotalValues(_pool);
      }

    case currentPoolData.PRIZE_POOL_TYPES.cream:
      {
        return await calculateCreamYieldTotalValues(_pool);
      }

    case currentPoolData.PRIZE_POOL_TYPES.genericYield:
      {
        switch ((_pool$prizePool$yield = _pool.prizePool.yieldSource) === null || _pool$prizePool$yield === void 0 ? void 0 : _pool$prizePool$yield.type) {
          case getCustomYieldSourceData.YIELD_SOURCES.aave:
            {
              return await calculateGenericYieldTotalValues(_pool);
            }

          case getCustomYieldSourceData.YIELD_SOURCES.sushi:
            {
              return await calculateGenericYieldTotalValues(_pool);
            }

          default:
            {
              return calculateClaimableYieldPrizeTotalValues(_pool);
            }
        }
      }

    case currentPoolData.PRIZE_POOL_TYPES.stake:
    default:
      {
        return calculateStakePrizeTotalValues(_pool);
      }
  }
};

const calculateCompoundYieldTotalValues = async _pool => {
  const pool = cloneDeep__default['default'](_pool);
  const cToken = pool.tokens.cToken;
  const underlyingToken = pool.tokens.underlyingToken;
  let compApy = '0';
  let yieldAmountUnformatted = pool.prize.amountUnformatted;
  const cTokenData = await fetch.fetch('https://yield.pooltogether-api.com/compound');
  const response = await cTokenData.json();

  if (cToken) {
    var _pool$prize, _pool$prize$yield, _pool$prize$yield$YIE, _pool$reserve;

    const cTokenResponse = response.cToken.find(token => token.token_address.toLowerCase() === cToken.address.toLowerCase());
    compApy = (cTokenResponse === null || cTokenResponse === void 0 ? void 0 : cTokenResponse.comp_supply_apy.value) || '0';
    let totalValueUsdScaled = utilities.calculatedEstimatedAccruedCompTotalValueUsdScaled(compApy, pool.tokens.ticket.totalValueUsdScaled.add(pool.tokens.sponsorship.totalValueUsdScaled), pool.prize.prizePeriodRemainingSeconds); // Add in the value of any unclaimed COMP

    if ((_pool$prize = pool.prize) !== null && _pool$prize !== void 0 && (_pool$prize$yield = _pool$prize.yield) !== null && _pool$prize$yield !== void 0 && (_pool$prize$yield$YIE = _pool$prize$yield[getCustomYieldSourceData.YIELD_SOURCES.comp]) !== null && _pool$prize$yield$YIE !== void 0 && _pool$prize$yield$YIE.unclaimedAmountUnformatted && pool.tokens.comp) {
      const unclaimedUsdAndAmountValues = calculateUsdValues(pool.prize.yield[getCustomYieldSourceData.YIELD_SOURCES.comp].unclaimedAmountUnformatted, pool.tokens.comp);
      totalValueUsdScaled = totalValueUsdScaled.add(unclaimedUsdAndAmountValues.totalValueUsdScaled);
    }

    const totalValueUsd = utilities.toNonScaledUsdString(totalValueUsdScaled);

    if (!pool.prize.yield) {
      pool.prize.yield = {
        [getCustomYieldSourceData.YIELD_SOURCES.comp]: {}
      };
    }

    pool.prize.yield = _rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2({}, pool.prize.yield), {}, {
      [getCustomYieldSourceData.YIELD_SOURCES.comp]: _rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2({}, pool.prize.yield[getCustomYieldSourceData.YIELD_SOURCES.comp]), {}, {
        totalValueUsd,
        totalValueUsdScaled
      })
    }); // Calculate yield

    yieldAmountUnformatted = utilities.calculateEstimatedCompoundPrizeWithYieldUnformatted(pool.prize.amountUnformatted, pool.tokens.ticket.totalSupplyUnformatted.add(pool.tokens.sponsorship.totalSupplyUnformatted), cToken.supplyRatePerBlock, pool.tokens.ticket.decimals, pool.prize.estimatedRemainingBlocksToPrize, (_pool$reserve = pool.reserve) === null || _pool$reserve === void 0 ? void 0 : _pool$reserve.rate);
  }

  const usdAndAmountValues = calculateUsdValues(yieldAmountUnformatted, underlyingToken);
  pool.prize.yield = _rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2({}, pool.prize.yield), usdAndAmountValues);
  return pool;
};

const calculateCreamYieldTotalValues = async _pool => {
  var _pool$reserve2;

  const pool = cloneDeep__default['default'](_pool);
  const response = await fetch.fetch('https://yield.pooltogether-api.com/cream');
  const creamMarketData = await response.json();
  const underlyingToken = pool.tokens.underlyingToken;
  const chainId = pool.config.chainId;
  const crToken = pool.tokens.crToken;

  const _ref = creamMarketData[chainId][crToken.address] || {
    supplyApy: 0
  },
        apr = _ref.supplyApy; // If there was an error fetching the apr


  if (!apr) {
    return pool;
  }

  const poolDepositsTotalSupplyUnformatted = pool.tokens.ticket.totalSupplyUnformatted.add(pool.tokens.sponsorship.totalSupplyUnformatted);
  const reserveRate = (_pool$reserve2 = _pool.reserve) === null || _pool$reserve2 === void 0 ? void 0 : _pool$reserve2.rate;
  const remainingSeconds = _pool.prize.prizePeriodRemainingSeconds;
  const estimatedUsdAndAmountValues = calculateEstimatedYieldUsdAndAmount(underlyingToken, apr, poolDepositsTotalSupplyUnformatted, reserveRate, remainingSeconds);
  const usdAndAmountValues = calculateUsdValues(estimatedUsdAndAmountValues.amountUnformatted.add(_pool.prize.amountUnformatted), underlyingToken);
  pool.prize.yield = _rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2({}, pool.prize.yield), usdAndAmountValues), {}, {
    estimatedPrize: _rollupPluginBabelHelpers.objectSpread2({}, estimatedUsdAndAmountValues)
  });
  return pool;
};

const calculateGenericYieldTotalValues = _pool => {
  var _pool$prizePool, _pool$prizePool$yield2, _pool$reserve3;

  const pool = cloneDeep__default['default'](_pool);
  const underlyingToken = _pool.tokens.underlyingToken;
  const apr = (_pool$prizePool = _pool.prizePool) === null || _pool$prizePool === void 0 ? void 0 : (_pool$prizePool$yield2 = _pool$prizePool.yieldSource) === null || _pool$prizePool$yield2 === void 0 ? void 0 : _pool$prizePool$yield2.apy; // If there was an error fetching the apy

  if (!apr) {
    return pool;
  }

  const poolDepositsTotalSupplyUnformatted = pool.tokens.ticket.totalSupplyUnformatted.add(pool.tokens.sponsorship.totalSupplyUnformatted);
  const reserveRate = (_pool$reserve3 = _pool.reserve) === null || _pool$reserve3 === void 0 ? void 0 : _pool$reserve3.rate;
  const remainingSeconds = _pool.prize.prizePeriodRemainingSeconds;
  const estimatedUsdAndAmountValues = calculateEstimatedYieldUsdAndAmount(underlyingToken, apr, poolDepositsTotalSupplyUnformatted, reserveRate, remainingSeconds);
  const usdAndAmountValues = calculateUsdValues(estimatedUsdAndAmountValues.amountUnformatted.add(_pool.prize.amountUnformatted), underlyingToken);
  pool.prize.yield = _rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2(_rollupPluginBabelHelpers.objectSpread2({}, pool.prize.yield), usdAndAmountValues), {}, {
    estimatedPrize: _rollupPluginBabelHelpers.objectSpread2({}, estimatedUsdAndAmountValues)
  });
  return pool;
};

const calculateEstimatedYieldUsdAndAmount = (underlyingToken, apr, poolDepositsTotalSupplyUnformatted, reserveRate, remainingSeconds) => {
  // Format to same decimal places, so we keep accuracy for floats
  const poolReserveRateUnformatted = ethers.ethers.utils.parseUnits(parseFloat(reserveRate).toFixed(Number(underlyingToken.decimals)), underlyingToken.decimals);
  const oneMinusPoolReserveRateUnformatted = ethers.ethers.utils.parseUnits('1', underlyingToken.decimals).sub(poolReserveRateUnformatted);
  const decimalsUnformatted = ethers.ethers.utils.parseUnits('1', underlyingToken.decimals);
  const apyUnformatted = ethers.ethers.utils.parseUnits(parseFloat(apr).toFixed(Number(underlyingToken.decimals)), underlyingToken.decimals);
  const underlyingTokensPerYearUnformatted = poolDepositsTotalSupplyUnformatted.mul(apyUnformatted).div(decimalsUnformatted);
  const underlyingTokensPerSecondNormalized = underlyingTokensPerYearUnformatted.mul(units.parseUnits('1', 18)) // Normalize
  .div(currentPoolData.SECONDS_PER_YEAR);
  let amountUnformatted;

  if (poolReserveRateUnformatted.isZero()) {
    amountUnformatted = underlyingTokensPerSecondNormalized.mul(remainingSeconds).div(units.parseUnits('1', 18)); // Denormalize
  } else if (poolReserveRateUnformatted.eq(ethers.ethers.constants.One)) {
    amountUnformatted = ethers.ethers.constants.Zero;
  } else {
    amountUnformatted = underlyingTokensPerSecondNormalized.mul(remainingSeconds).mul(oneMinusPoolReserveRateUnformatted).div(decimalsUnformatted).div(units.parseUnits('1', 18)); // Denormalize
  }

  return calculateUsdValues(amountUnformatted, underlyingToken);
}; // const calculateAaveYieldExtendedValues = (_pool, fetch) => {
// const pool = cloneDeep(_pool)
// TODO: Use for additional tokens aave drips
// const aaveAdditionalApy = _pool.prizePool.yieldSource.aave.additionalApy
// TODO: Use for additional tokens aave drips
// const aaveTotalValueUsd = null
// const aaveTotalValueUsdScaled = null
// pool.prize.yield = {
// ...pool.prize.yield,
// TODO: Use for additional tokens aave drips
// [YIELD_SOURCES.aave]: {
//   totalValueUsd: aaveTotalValueUsd,
//   totalValueUsdScaled: aaveTotalValueUsdScaled
// }
// }
//   return pool
// }


const calculateStakePrizeTotalValues = _pool => {
  const pool = cloneDeep__default['default'](_pool);
  const usdAndAmountValues = calculateUsdValues(_pool.prize.amountUnformatted, _pool.tokens.underlyingToken);
  pool.prize.stake = _rollupPluginBabelHelpers.objectSpread2({}, usdAndAmountValues);
  return pool;
};

const calculateClaimableYieldPrizeTotalValues = _pool => {
  const pool = cloneDeep__default['default'](_pool);
  const usdAndAmountValues = calculateUsdValues(_pool.prize.amountUnformatted, _pool.tokens.underlyingToken);
  pool.prize.yield = _rollupPluginBabelHelpers.objectSpread2({}, usdAndAmountValues);
  return pool;
};

const calculateUsdValues = (amountUnformatted, underlyingToken) => {
  const amount = units.formatUnits(amountUnformatted, underlyingToken.decimals);
  const totalUsdValueUnformatted = utilities.amountMultByUsd(amountUnformatted, underlyingToken.usd);
  const totalValueUsd = units.formatUnits(totalUsdValueUnformatted, underlyingToken.decimals);
  const totalValueUsdScaled = utilities.toScaledUsdBigNumber(totalValueUsd);
  return {
    amountUnformatted,
    amount,
    totalValueUsd,
    totalValueUsdScaled
  };
};

exports.calculateYieldTotalValuesUsd = calculateYieldTotalValuesUsd;
